#include <Wire.h>
#include "Adafruit_SGP30.h" 
#include <DHT.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define DHTPIN 0
#define DHTTYPE DHT11   // DHT 11
DHT dht(DHTPIN, DHTTYPE);

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

Adafruit_SGP30 sgp;

uint32_t getAbsoluteHumidity(float temperature, float humidity) {
  // approximation formula from Sensirion SGP30 Driver Integration chapter 3.15
  const float absoluteHumidity = 216.7f * ((humidity / 100.0f) * 6.112f * exp((17.62f * temperature) / (243.12f + temperature)) / (273.15f + temperature)); // [g/m^3]
  const uint32_t absoluteHumidityScaled = static_cast<uint32_t>(1000.0f * absoluteHumidity); // [mg/m^3]
  return absoluteHumidityScaled;
}

void setup() {
  Serial.begin(115200);
  /*while (!Serial) {
    delay(10);  // Wait for serial console to open!
  }*/
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { 
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }

  // Show initial display buffer contents on the screen --
  // the library initializes this with an Adafruit splash screen.
  display.display();
  //delay(2000); // Pause for 2 seconds

  // Clear the buffer
  display.clearDisplay();

  dht.begin(); 
  Serial.println("SGP30 test");

  if (! sgp.begin()) {
    Serial.println("Sensor not found :(");
    while (1);
  }
  Serial.print("Found SGP30 serial #");
  Serial.print(sgp.serialnumber[0], HEX);
  Serial.print(sgp.serialnumber[1], HEX);
  Serial.println(sgp.serialnumber[2], HEX);
 

  
  // If you have a baseline measurement from before you can assign it to start, to 'self-calibrate'
  //sgp.setIAQBaseline(0x8E68, 0x8F41);  // Will vary for each sensor!
}
int counter = 0;





void loop() {
  // If you have a temperature / humidity sensor, you can set the absolute humidity to enable the humditiy compensation for the air quality signals
  //float temperature = 22.1; // [°C]
  //float humidity = 45.2; // [%RH]
  //sgp.setHumidity(getAbsoluteHumidity(temperature, humidity));
 float h = dht.readHumidity();
 float t = dht.readTemperature(); // Read temperature as Celsius (the default)
 float f = dht.readTemperature(true); // Read temperature as Fahrenheit (isFahrenheit = true)

 if (isnan(h) || isnan(t) || isnan(f)) {
   Serial.println("Failed to read from DHT sensor!");
   return;
 }
    Serial.print( "Temperature (C)= " );
    Serial.println( t);
    Serial.print( " - Temperature (F)= " );
    Serial.println(f);
    Serial.print( " - Humidity = " );
    Serial.print( h);
    Serial.println( "%" );
    Serial.println( "");



  if (! sgp.IAQmeasure()) {
    Serial.println("Measurement failed");
    return;
  }

  Serial.print("TVOC "); Serial.print(sgp.TVOC); Serial.println(" ppb\t");
  Serial.print("eCO2 "); Serial.print(sgp.eCO2); Serial.println(" ppm");

  if (! sgp.IAQmeasureRaw()) {
    Serial.println("Raw Measurement failed");
    return;
  }
  Serial.print("Raw H2 "); Serial.print(sgp.rawH2); Serial.println(" \t");
  Serial.print("Raw Ethanol "); Serial.print(sgp.rawEthanol); Serial.println("");
  Serial.println(" ");

  display.setCursor(20, 0); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print("Weather station");
  
  display.setCursor(10, 16); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print(t);
  display.print(" C"); 

  display.setCursor(70, 16); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print(h);
  display.print(" %"); 
  
  display.setCursor(10, 26); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print("CO2:");
  display.print(sgp.eCO2);
  display.print(" ppm");
 
  display.setCursor(10, 36); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print("H2:");
  display.print(sgp.rawH2);
 
  display.setCursor(10, 46); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print("EtOH:");
  display.print(sgp.rawEthanol);
 
  display.setCursor(10, 56); //oled display
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.print("TVOC:");
  display.print(sgp.TVOC);
  display.print(" ppb");

  display.display();
  display.clearDisplay();

  delay(1000);

  counter++;
  if (counter == 30) {
    counter = 0;

    uint16_t TVOC_base, eCO2_base;
    if (! sgp.getIAQBaseline(&eCO2_base, &TVOC_base)) {
      Serial.println("Failed to get baseline readings");
      return;
    }
    Serial.print("****Baseline values: eCO2: 0x"); Serial.print(eCO2_base, HEX);
    Serial.print(" & TVOC: 0x"); Serial.println(TVOC_base, HEX);
  }
}
